# -*- coding: utf-8 -*-
"""
    version

    Store the version separately here so that the application can
    expose the version and setup.py can resuse this version without
    loading the package through __init__ module.

    :copyright: © 2013 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
__version__ = '0.5.0'
